#!/usr/bin/perl -w -I. -I/usr/sausalito/perl/ -I/usr/sausalito/handlers/base/frontpage/

use strict;
umask(002);

use CCE;
use Sauce::Validators qw( password );

my $cce = new CCE( Namespace => "Frontpage",
                   Domain    => "base-frontpage" );

my $homeGroup = 'home';
my $home_rootweb = '/usr/local/frontpage/we80.cnf';
my $htPassword = '/usr/bin/htpasswd';
my $home_basedir = '/home/groups/home';
my $home_htpwd = "$home_basedir/web/_vti_pvt/service.pwd"; 
my $fp_webmaster = 'webmaster';
my $httpd_conf = '/etc/httpd/conf/httpd.conf';
my $httpd_access = '/etc/httpd/conf/access.conf';
my $Fpx_admsrvexe = '/usr/local/frontpage/currentversion/bin/fpsrvadm.exe';
my $Lockdir = '/etc/locks';
my $Fpx_fpexec = '/usr/sbin/fpexec';

$cce->connectfd();

my $fpx = $cce->event_object();

if( fpx_update($cce, $fpx) ) {
        $cce->bye("SUCCESS");
        exit 0;
} else {
        $cce->bye("FAIL");
        exit 1;
}

sub fpx_update {
	my $cce = shift;
	my $fpx = shift;

	if( $fpx->{enabled} && fpx_get_homeweb() ) {
		fpx_set_homepasswordWebmaster( $fpx->{passwordWebmaster} ) || return 0; 
        } elsif( $fpx->{enabled} ) {
		fpx_enable_home( $fpx->{passwordWebmaster} ) || return 0;
	} else {
		fpx_disable_home() || return 0;
	}

	return 1;	
}

sub fpx_enable_home {
	my $passwordWebmaster = shift;

	my($user)='nobody';
	my $could_not_install;

	# Call fpadmsrv.exe (ugh)
	system("$Fpx_admsrvexe -o install -p 80 -t apache-fp -m \"\" -u \"webmaster\" -pw \"$passwordWebmaster\" -s \"$httpd_conf\" -xu \"$user\" -xg \"$homeGroup\" >> /var/log/fpx.log") && ($could_not_install = $!);

	# Bail!
	if ($could_not_install) {
		$cce->warn('fpx_failed');
		return 0;
	}

	# Update Apache access levels (pooch security)
	fpx_change_access($home_basedir,1);

	# Add SMTP capability to FPX forms
      	if (-w $home_rootweb) {
        	my($orig_cnf);
		open(CNF, $home_rootweb);
		while(<CNF>) { $orig_cnf .= $_; }
		close(CNF);

		unless($orig_cnf =~ /smtphost/i) {
			open(NUCNF,">> $home_rootweb");
			print NUCNF "SMTPHost: 127.0.0.1\n";
			close(NUCNF);
		}

        # Must be readable by nobody
        chmod(0644,$home_rootweb);
	}

	return 1;
}

sub fpx_disable_home {
	system("/bin/rm $home_rootweb > /dev/null 2>&1; /bin/rm -rf `/usr/bin/find $home_basedir -name \"_vti_*\"` > /dev/null 2>&1");
	
	# fpadmsrv chown -R's web/ instead of web/* so we need to patch ownership of the web dir back to root
	chown( (getpwnam('root'))[2], (getgrnam('home'))[2], "$home_basedir/web");
	chmod(02775, "$home_basedir/web");

	return 1;
}

sub fpx_get_homeweb {
	return 1 if( -e $home_rootweb);
	return 0;
}

sub fpx_set_homepasswordWebmaster {
	my $passwordWebmaster = shift;

	system("$htPassword -b $home_htpwd $fp_webmaster $passwordWebmaster > /dev/null 2>&1") && return 0;
	return 1;
} 
	
sub fpx_change_access {
	my( $basedir, $status ) = @_;

    	my ($inentry, $found_web, $conf);
    	my ($on,$ret);

	# watch locks
	sleep 1 if (-e '/etc/httpd/conf/access.conf');
	sleep 1 if (-e '/etc/httpd/conf/access.conf');
	sleep 1 if (-e '/etc/httpd/conf/access.conf');
	return 0 if (-e '/etc/httpd/conf/access.conf');

    	# We only AllowOverride All and Options All sites/groups that have frontpage enabled.
    	# You're only shooting yourself in the foot if you're into that sort of thing.

    	# Track first/last site FP usage to tweak systee
    	# security.  No need to have suid 0 unless we're using it
    	my($other_use) = 0;

    	# update the current access configuration
    	open(HTAX,$httpd_access) ||
        	return 0;

    	open (HTCTMP, ">$Lockdir/access.conf") ||
        	return 0;

    	$found_web = $inentry = 0;
    	# fix the httpd.conf file
    	while (<HTAX>) {
		if (/^<Directory $basedir/ ... /^<\/Directory/) {
			$found_web = 1;
			print HTCTMP if ($status);
			next;
		} else {
			$other_use = 1 if (/AllowOverride All/);
            	}
        	print HTCTMP;
	}
	close (HTAX);

    
	if (!$found_web && $status) {
        	print HTCTMP <<EOF;
<Directory $basedir>
AllowOverride All
Options All
</Directory>
EOF
        }
	close (HTCTMP);

	rename("$Lockdir/access.conf", $httpd_access) ||
		return vsite_abort(MSG_error("4585"));
    
	# Enable/disable suid 0 access
	if ($status) {
		chmod(04755, $Fpx_fpexec);
	} elsif (!$other_use) {
		chmod(00755, $Fpx_fpexec);
        }
    
	return 1;
}
